@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  *
- * $Id: scim_hangul_imengine.cpp,v 1.32 2007/01/06 14:00:57 hwanjin Exp $
+ * $Id: scim_hangul_imengine.cpp,v 1.34 2007/05/27 13:08:07 hwanjin Exp $
  */
 
 #define Uses_SCIM_UTILITY
@@ -151,6 +151,17 @@ HangulFactory::HangulFactory (const ConfigPointer &config)
     m_commit_by_word = false;
 
     m_hanja_table = hanja_table_load(NULL);
+    m_symbol_table = NULL;
+
+    std::string symbol_file = getenv("HOME");
+    symbol_file += "/.scim/hangul/symbol.txt";
+    if (access(symbol_file.c_str(), R_OK) == 0)
+	m_symbol_table = hanja_table_load(symbol_file.c_str());
+    if (m_symbol_table == NULL) {
+	symbol_file = SCIM_HANGUL_DATADIR "/symbol.txt";
+	if (access(symbol_file.c_str(), R_OK) == 0)
+	    m_symbol_table = hanja_table_load(symbol_file.c_str());
+    }
 
     set_languages ("ko");
 
@@ -424,13 +435,6 @@ HangulInstance::process_key_event (const KeyEvent& rawkey)
 	toggle_hanja_mode();
     }
 
-    /* change to ascii mode on ESCAPE key, for vi users */
-    if (use_ascii_mode() && !is_hanja_mode()) {
-	if (key.code == SCIM_KEY_Escape) {
-	    toggle_hangul_mode();
-	}
-    }
-
     /* toggle candidate table */
     if (is_hanja_key (key)) {
 	if (is_hanja_mode()) {
@@ -461,6 +465,16 @@ HangulInstance::process_key_event (const KeyEvent& rawkey)
 	    return true;
     }
 
+    /* change to ascii mode on ESCAPE key, for vi users.
+     * We should process this key after processing candidate keys,
+     * or input mode will be changed to non-hangul mode when the user presses
+     * escape key to close candidate window. */
+    if (use_ascii_mode() && !is_hanja_mode()) {
+	if (key.code == SCIM_KEY_Escape) {
+	    toggle_hangul_mode();
+	}
+    }
+
     /* backspace */
     if (is_backspace_key(key)) {
         bool ret = hangul_ic_backspace(m_hic);
@@ -717,42 +731,55 @@ HangulInstance::get_candidate_string()
 void
 HangulInstance::update_candidates ()
 {
-    String str = get_candidate_string();
-
-    SCIM_DEBUG_IMENGINE(1) << "candidate string: " << str << "\n";
-
     m_lookup_table.clear ();
     m_candidate_comments.clear ();
 
-    if (str.length() > 0) {
-	HanjaList* list;
-	if (is_hanja_mode() || m_factory->m_commit_by_word) {
-	    list = hanja_table_match_prefix(m_factory->m_hanja_table,
-						str.c_str());
-	} else {
-	    list = hanja_table_match_suffix(m_factory->m_hanja_table,
-						str.c_str());
+    HanjaList* list = NULL;
+
+    // search for symbol character
+    // key string for symbol character is like:
+    //  'ㄱ', 'ㄴ', 'ㄷ', etc
+    WideString preeditw = get_preedit_string();
+    if (preeditw.length() == 1) {
+	String key = utf8_wcstombs(preeditw);
+	list = hanja_table_match_suffix(m_factory->m_symbol_table, key.c_str());
+    }
+
+    // search for hanja
+    if (list == NULL) {
+	String str = get_candidate_string();
+	SCIM_DEBUG_IMENGINE(1) << "candidate string: " << str << "\n";
+
+	if (str.length() > 0) {
+	    if (is_hanja_mode() || m_factory->m_commit_by_word) {
+		list = hanja_table_match_prefix(m_factory->m_hanja_table,
+						    str.c_str());
+	    } else {
+		list = hanja_table_match_suffix(m_factory->m_hanja_table,
+						    str.c_str());
+	    }
+	}
+    } 
+    
+    if (list != NULL) {
+	int n = hanja_list_get_size(list);
+	for (int i = 0; i < n; ++i) {
+	    const char* value = hanja_list_get_nth_value(list, i);
+	    const char* comment = hanja_list_get_nth_comment(list, i);
+	    WideString candidate = utf8_mbstowcs(value, -1);
+	    m_lookup_table.append_candidate(candidate);
+	    m_candidate_comments.push_back(String(comment));
 	}
-        if (list != NULL) {
-	    int n = hanja_list_get_size(list);
-            for (int i = 0; i < n; ++i) {
-		const char* value = hanja_list_get_nth_value(list, i);
-		const char* comment = hanja_list_get_nth_comment(list, i);
-		WideString candidate = utf8_mbstowcs(value, -1);
-                m_lookup_table.append_candidate(candidate);
-                m_candidate_comments.push_back(String(comment));
-            }
 
-            m_lookup_table.set_page_size (9);
-            m_lookup_table.show_cursor ();
+	m_lookup_table.set_page_size (9);
+	m_lookup_table.show_cursor ();
 
-            update_lookup_table (m_lookup_table);
-            show_lookup_table ();
+	update_lookup_table (m_lookup_table);
+	show_lookup_table ();
 
-            hangul_update_aux_string ();
+	hangul_update_aux_string ();
 
-	    hanja_list_delete(list);
-        }
+	hanja_list_delete(list);
     }
 
     if (m_lookup_table.number_of_candidates() <= 0) {
